@@ -3,7 +3,7 @@ import warnings
 from relevanceai import vector_tools
 from relevanceai.http_client import Client
 
-# Cluster Base Utilities
+# Cluster _Base Utilities
 from relevanceai.clusterer.cluster_base import ClusterBase, CentroidClusterBase
 from relevanceai.clusterer.clusterer import ClusterOps
 import requests
@@ -14,7 +14,7 @@ try:
 except ModuleNotFoundError:
     pass
 
-__version__ = "0.33.2"
+__version__ = "1.1.4"
 
 try:
     pypi_data = requests.get("https://pypi.org/pypi/relevanceai/json").json()
@@ -25,10 +25,17 @@ try:
         latest_version = None
 
     if __version__ != latest_version and latest_version is not None:
-        warnings.warn(
-            "Your RelevanceAI version ({version}) is not the latest. Please install the latest version ({latest_version}) by running pip install -U relevanceai.".format(
-                version=__version__, latest_version=latest_version
-            )
+        changelog_url: str = (
+            f"https://relevanceai.readthedocs.io/en/{__version__}/changelog.html"
         )
+        MESSAGE = """We noticed you don't have the latest version! 
+We recommend updating to the latest version ({latest_version}) to get all bug fixes and newest features!
+You can do this by running pip install -U relevanceai. 
+Changelog: {changelog_url}.""".format(  # type: ignore
+            version=__version__,
+            latest_version=latest_version,
+            changelog_url=changelog_url,
+        )
+        warnings.warn(MESSAGE)
 except:
     pass
