@@ -7,11 +7,13 @@ You can run the ClusterOps as such:
 .. code-block::
 
     from relevanceai import Client
-    from relevanceai.clusterer import KMeansModel
     client = Client()
-    clusterops = client.ClusterOps(model, alias="kmeans_2")
-    df = client.Dataset("_github_repo_vectorai")
-    clusterer.fit_predict_update(df, ["documentation_vector_"])
+    df = client.Dataset("sample_dataset")
+
+    from sklearn.cluster import KMeans
+    model = KMeans(n_clusters=2)
+    cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+    cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
 
 You can view other examples of how to interact with this class here :ref:`integration`.
 
@@ -31,14 +33,19 @@ from relevanceai.clusterer.cluster_base import (
     BatchClusterBase,
 )
 
+from relevanceai.analytics_funcs import track
+
 # We use the second import because the first one seems to be causing errors with isinstance
 # from relevanceai.dataset_api import Dataset
-from relevanceai.integration_checks import is_sklearn_available
+from relevanceai.integration_checks import is_sklearn_available, is_hdbscan_available
 from relevanceai.dataset_api.cluster_groupby import ClusterGroupby, ClusterAgg
 from relevanceai.dataset_api import Dataset
+from relevanceai.errors import NoDocumentsError
 
 from doc_utils import DocUtils
 
+from relevanceai.vector_tools.cluster import Cluster
+
 SILHOUETTE_INFO = """
 Good clusters have clusters which are highly seperated and elements within which are highly cohesive. <br/>
 <b>Silohuette Score</b> is a metric from <b>-1 to 1</b> that calculates the average cohesion and seperation of each element, with <b>1</b> being clustered perfectly, <b>0</b> being indifferent and <b>-1</b> being clustered the wrong way"""
@@ -79,24 +86,25 @@ class ClusterOps(BatchAPIClient):
     .. code-block::
 
         from relevanceai import Client
-        from relevanceai.clusterer import KMeansModel
+        client = Client()
+        df = client.Dataset("sample_dataset")
 
+        from sklearn.cluster import KMeans
         model = KMeans(n_clusters=2)
-        clusterer = client.ClusterOps(model, alias="kmeans_2")
-
-        df = client.Dataset("sample")
-        clusterer.fit(df, vector_fields=["sample_vector_"])
+        cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+        cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
 
     """
 
     def __init__(
         self,
         alias: str,
+        project: str,
+        api_key: str,
+        firebase_uid: str,
         model: Union[BatchClusterBase, ClusterBase, CentroidClusterBase] = None,
         dataset_id: Optional[str] = None,
         vector_fields: Optional[List[str]] = None,
-        project: Union[str, None] = None,
-        api_key: Union[str, None] = None,
         cluster_field: str = "_cluster_",
     ):
         self.alias = alias
@@ -107,6 +115,7 @@ class ClusterOps(BatchAPIClient):
             )
 
         self.model = self._assign_model(model)
+        self.firebase_uid = firebase_uid
 
         if dataset_id is not None:
             self.dataset_id: str = dataset_id
@@ -119,24 +128,27 @@ class ClusterOps(BatchAPIClient):
             self.project: str = project
             self.api_key: str = api_key
 
-        super().__init__(project=project, api_key=api_key)
+        super().__init__(project=project, api_key=api_key, firebase_uid=firebase_uid)
 
     def __call__(self):
         self.groupby = ClusterGroupby(
-            self.project,
-            self.api_key,
-            self.dataset_id,
+            project=self.project,
+            api_key=self.api_key,
+            dataset_id=self.dataset_id,
+            firebase_uid=self.firebase_uid,
             alias=self.alias,
             vector_fields=self.vector_fields,
         )
         self.agg = ClusterAgg(
-            self.project,
-            self.api_key,
-            self.dataset_id,
+            project=self.project,
+            api_key=self.api_key,
+            dataset_id=self.dataset_id,
+            firebase_uid=self.firebase_uid,
             vector_fields=self.vector_fields,
             alias=self.alias,
         )
 
+    @track
     def groupby(self):
         return ClusterGroupby(
             project=self.project,
@@ -146,12 +158,14 @@ class ClusterOps(BatchAPIClient):
             vector_fields=self.vector_fields,
         )
 
+    @track
     def agg(self, groupby_call):
         """Aggregate the cluster class."""
         self.groupby = ClusterGroupby(
-            self.project,
-            self.api_key,
-            self.dataset_id,
+            project=self.project,
+            api_key=self.api_key,
+            dataset_id=self.dataset_id,
+            firebase_uid=self.firebase_uid,
             alias=self.alias,
             vector_fields=self.vector_fields,
         )
@@ -159,17 +173,42 @@ class ClusterOps(BatchAPIClient):
             project=self.project,
             api_key=self.api_key,
             dataset_id=self.dataset_id,
+            firebase_uid=self.firebase_uid,
             vector_fields=self.vector_fields,
             alias=self.alias,
-            groupby_call=groupby_call,
         )
 
     # Adding first-class sklearn integration
     def _assign_sklearn_model(self, model):
         # Add support for not just sklearn models but sklearn models
         # with first -class integration for kmeans
-        from sklearn.cluster import KMeans, MiniBatchKMeans
+        from sklearn.cluster import (
+            KMeans,
+            MiniBatchKMeans,
+            DBSCAN,
+            Birch,
+            SpectralClustering,
+            OPTICS,
+            AgglomerativeClustering,
+            AffinityPropagation,
+            MeanShift,
+            FeatureAgglomeration,
+        )
 
+        POSSIBLE_MODELS = [
+            SpectralClustering,
+            Birch,
+            DBSCAN,
+            OPTICS,
+            AgglomerativeClustering,
+            AffinityPropagation,
+            MeanShift,
+            FeatureAgglomeration,
+        ]
+        if is_hdbscan_available():
+            from hdbscan import HDBSCAN
+
+            POSSIBLE_MODELS.append(HDBSCAN)
         if model.__class__ == KMeans:
 
             class CentroidClusterModel(CentroidClusterBase):
@@ -202,21 +241,45 @@ class ClusterOps(BatchAPIClient):
 
             new_model = BatchCentroidClusterModel(model)
             return new_model
-        if hasattr(model, "fit_documents"):
+
+        elif isinstance(model, tuple(POSSIBLE_MODELS)):
+
+            class CentroidClusterModel(CentroidClusterBase):
+                def __init__(self, model):
+                    self.model: Union[SpectralClustering, Birch, DBSCAN] = model
+
+                def fit_predict(self, X):
+                    self._X = np.array(X)
+                    self._labels = self.model.fit_predict(X)
+                    return self._labels
+
+                def get_centers(self):
+                    # Get the centers for each label
+                    centers = []
+                    for l in sorted(np.unique(self._labels).tolist()):
+                        centers.append(self._X[self._labels == l].mean(axis=0).tolist())
+                    return centers
+
+            new_model = CentroidClusterModel(model)
+            return new_model
+        elif hasattr(model, "fit_documents"):
             return model
-        elif hasattr(model, "fit_transform"):
-            data = {"fit_transform": model.fit_transform, "metadata": model.__dict__}
+        elif hasattr(model, "fit_predict"):
+            data = {"fit_predict": model.fit_predict, "metadata": model.__dict__}
             ClusterModel = type("ClusterBase", (ClusterBase,), data)
             return ClusterModel()
-        elif hasattr(model, "fit_predict"):
-            data = {"fit_transform": model.fit_predict, "metadata": model.__dict__}
+        elif hasattr(model, "fit_transform"):
+            data = {"fit_predict": model.fit_transform, "metadata": model.__dict__}
             ClusterModel = type("ClusterBase", (ClusterBase,), data)
             return ClusterModel()
 
     def _assign_model(self, model):
         # Check if this is a model that will fit
         # otherwise - forces a Clusterbase
-        if is_sklearn_available() and "sklearn" in str(type(model)):
+        if (is_sklearn_available() or is_hdbscan_available()) and (
+            "sklearn" in str(type(model)).lower()
+            or "hdbscan" in str(type(model)).lower()
+        ):
             model = self._assign_sklearn_model(model)
             if model is not None:
                 return model
@@ -225,11 +288,11 @@ class ClusterOps(BatchAPIClient):
             return model
         elif hasattr(model, "fit_documents"):
             return model
-        elif hasattr(model, "fit_predict"):
-            # Support for SKLEARN interface
-            data = {"fit_predict": model.fit_transform, "metadata": model.__dict__}
-            ClusterModel = type("ClusterBase", (ClusterBase,), data)
-            return ClusterModel()
+        # elif hasattr(model, "fit_predict"):
+        #     # Support for SKLEARN interface
+        #     data = {"fit_predict": model.fit_predict, "metadata": model.__dict__}
+        #     ClusterModel = type("ClusterBase", (ClusterBase,), data)
+        #     return ClusterModel()
         elif hasattr(model, "fit_predict"):
             data = {"fit_predict": model.fit_predict, "metadata": model.__dict__}
             ClusterModel = type("ClusterBase", (ClusterBase,), data)
@@ -238,26 +301,59 @@ class ClusterOps(BatchAPIClient):
             return model
         raise TypeError("Model should be inherited from ClusterBase.")
 
-    def _token_to_auth(self):
+    def _token_to_auth(self, token=None):
         SIGNUP_URL = "https://cloud.relevance.ai/sdk/api"
-        if not os.path.exists(self._cred_fn):
-            # We repeat it twice because of different behaviours
-            print(f"Authorization token (you can find it here: {SIGNUP_URL} )")
-            token = getpass.getpass(f"Auth token:")
-            project = token.split(":")[0]
-            api_key = token.split(":")[1]
-            self._write_credentials(project, api_key)
+
+        if os.path.exists(self._cred_fn):
+            credentials = self._read_credentials()
+            return credentials
+
+        elif token:
+            return self._process_token(token)
+
         else:
-            data = self._read_credentials()
-            project = data["project"]
-            api_key = data["api_key"]
-        return project, api_key
+            print(f"Activation token (you can find it here: {SIGNUP_URL} )")
+            if not token:
+                token = getpass.getpass(f"Activation token:")
+            return self._process_token(token)
+
+    def _process_token(self, token: str):
+        split_token = token.split(":")
+        project = split_token[0]
+        api_key = split_token[1]
+        if len(split_token) > 2:
+            region = split_token[3]
+            base_url = self._region_to_url(region)
+
+            if len(split_token) > 3:
+                firebase_uid = split_token[4]
+                return self._write_credentials(
+                    project=project,
+                    api_key=api_key,
+                    base_url=base_url,
+                    firebase_uid=firebase_uid,
+                )
+
+            else:
+                return self._write_credentials(
+                    project=project, api_key=api_key, base_url=base_url
+                )
+
+        else:
+            return self._write_credentials(project=project, api_key=api_key)
 
     def _read_credentials(self):
         return json.load(open(self._cred_fn))
 
-    def _write_credentials(self, project, api_key):
-        json.dump({"project": project, "api_key": api_key}, open(self._cred_fn, "w"))
+    def _write_credentials(self, **kwargs):
+        print(
+            f"Saving credentials to {self._cred_fn}. Remember to delete this file if you do not want credentials saved."
+        )
+        json.dump(
+            kwargs,
+            open(self._cred_fn, "w"),
+        )
+        return kwargs
 
     def _init_dataset(self, dataset):
         if isinstance(dataset, Dataset):
@@ -271,6 +367,7 @@ class ClusterOps(BatchAPIClient):
                 "Dataset type needs to be either a string or Dataset instance."
             )
 
+    @track
     def list_closest_to_center(
         self,
         dataset: Optional[Union[str, Dataset]] = None,
@@ -331,13 +428,14 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample_dataset")
+            df = client.Dataset("sample_dataset_id")
 
-            from relevanceai.clusterer import KMeansModel
-            kmeans = KMeans(n_clusters=5)
-            clusterer = client.ClusterOps(kmeans)
-            clusterer.fit(df, ["sample_vector_"])
-            clusterer.list_closest_to_center()
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+            cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
+
+            cluster_ops.list_closest_to_center()
 
         """
         return self.datasets.cluster.centroids.list_closest_to_center(
@@ -360,9 +458,10 @@ class ClusterOps(BatchAPIClient):
             include_count=include_count,
         )
 
+    @track
     def aggregate(
         self,
-        dataset: Optional[Union[str, Dataset]],
+        vector_fields: List[str] = None,
         metrics: list = [],
         sort: list = [],
         groupby: list = [],
@@ -371,6 +470,7 @@ class ClusterOps(BatchAPIClient):
         page: int = 1,
         asc: bool = False,
         flatten: bool = True,
+        dataset: Optional[Union[str, Dataset]] = None,
     ):
         """
         Takes an aggregation query and gets the aggregate of each cluster in a collection. This helps you interpret each cluster and what is in them.
@@ -484,23 +584,27 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample_dataset")
+            df = client.Dataset("sample_dataset_id")
+
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+            cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
 
-            from relevanceai.clusterer import KMeansModel
-            clusterer = client.ClusterOps(5)
-            clusterer.fit(df, ["sample_vector_"])
             clusterer.aggregate(
-                groupby=[],
-                metrics=[
-                    {"name": "average_score", "field": "final_score", "agg": "avg"},
-                ]
+                "sample_dataset_id",
+                groupby=[{
+                    "field": "title",
+                    "agg": "wordcloud",
+                }],
+                vector_fields=['sample_vector_']
             )
 
 
         """
         return self.services.cluster.aggregate(
             dataset_id=self._retrieve_dataset_id(dataset),
-            vector_fields=self.vector_fields,
+            vector_fields=self.vector_fields if not vector_fields else vector_fields,
             groupby=groupby,
             metrics=metrics,
             sort=sort,
@@ -553,9 +657,14 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample")
-            clusterer = df.auto_cluster("kmeans-3", ["sample_vector_"])
-            clusterer.list_furthest_from_center()
+            df = client.Dataset("sample_dataset")
+
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+            cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
+
+            cluster_ops.list_furthest_from_center()
 
         """
         return self.datasets.cluster.centroids.list_furthest_from_center(
@@ -568,10 +677,10 @@ class ClusterOps(BatchAPIClient):
 
     def _insert_centroid_documents(self):
         if hasattr(self.model, "get_centroid_documents"):
+            print("Inserting centroid documents...")
             centers = self.get_centroid_documents()
 
             if hasattr(self.model, "get_centers"):
-                center_vectors = self.model.get_centers()
                 centers = self.get_centroid_documents()
 
             # Change centroids insertion
@@ -583,13 +692,6 @@ class ClusterOps(BatchAPIClient):
             )
             self.logger.info(results)
 
-            self.datasets.cluster.centroids.list_closest_to_center(
-                self.dataset_id,
-                vector_fields=self.vector_fields,
-                alias=self.alias,
-                centroid_vector_fields=self.vector_fields,
-                page_size=20,
-            )
         return
 
     def _retrieve_dataset_id(self, dataset: Optional[Union[str, Dataset]]) -> str:
@@ -610,6 +712,7 @@ class ClusterOps(BatchAPIClient):
                 raise ValueError("Please supply dataset.")
         return dataset_id
 
+    @track
     def insert_centroid_documents(
         self, centroid_documents: List[Dict], dataset: Union[str, Dataset] = None
     ):
@@ -631,10 +734,15 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            clusterer = client.ClusterOps(alias="example")
-            # available if you inherit from centroidbase
-            centroids = model.get_centroid_documents()
-            clusterer.insert_centroid_documents(centroids)
+            df = client.Dataset("sample_dataset")
+
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+            cluster_ops.fit_predict_update(df, vector_fields=["sample_vector_"])
+
+            centroids = cluster_ops.get_centroid_documents()
+            cluster_ops.insert_centroid_documents(centroids)
 
         """
 
@@ -646,6 +754,7 @@ class ClusterOps(BatchAPIClient):
         )
         return results
 
+    @track
     def get_centroid_documents(self) -> List:
         """
         Get the centroid documents to store. This enables you to use `list_closest_to_center()`
@@ -698,7 +807,7 @@ class ClusterOps(BatchAPIClient):
         """
         See your centroids if there are any.
         """
-        return self.services.cluster.centroids.list(
+        return self.services.cluster.centroids.documents(
             self.dataset_id,
             vector_fields=self.vector_fields,
             alias=self.alias,
@@ -707,6 +816,7 @@ class ClusterOps(BatchAPIClient):
             include_vector=True,
         )
 
+    @track
     def delete_centroids(self, dataset: Union[str, Dataset], vector_fields: List):
         """Delete the centroids after clustering."""
         # TODO: Fix delete centroids once its moved over to Node JS
@@ -724,8 +834,13 @@ class ClusterOps(BatchAPIClient):
         )
         return response.json()["status"]
 
+    @track
     def fit_predict_update(
-        self, dataset: Union[Dataset, str], vector_fields: List, filters: List = []
+        self,
+        dataset: Union[Dataset, str],
+        vector_fields: List,
+        filters: List = [],
+        include_grade: bool = False,
     ):
         """
         This function fits a cluster model onto a dataset. It sits under `fit`
@@ -749,19 +864,20 @@ class ClusterOps(BatchAPIClient):
         .. code-block::
 
             from relevanceai import ClusterBase, Client
-            import random
             client = Client()
 
+            import random
             class CustomClusterModel(ClusterBase):
                 def fit_predict(self, X):
                     cluster_labels = [random.randint(0, 100) for _ in range(len(X))]
                     return cluster_labels
 
             model = CustomClusterModel()
-            clusterer = client.ClusterOps(model, alias="random")
-            df = client.Dataset("_github_repo_vectorai")
 
-            clusterer.fit_predict_update(df, vector_fields=["documentation_vector_"])
+            clusterer = client.ClusterOps(model, alias="random_clustering")
+            df = client.Dataset("sample_dataset")
+
+            clusterer.fit_predict_update(df, vector_fields=["sample_vector_"])
 
         """
 
@@ -788,12 +904,15 @@ class ClusterOps(BatchAPIClient):
             dataset_id=self.dataset_id, filters=filters, select_fields=vector_fields
         )
 
+        if len(docs) == 0:
+            raise NoDocumentsError()
         print("Fitting and predicting on all documents")
         clustered_docs = self.fit_predict_documents(
             vector_fields,
             docs,
             return_only_clusters=True,
             inplace=False,
+            include_grade=include_grade,
         )
 
         # Updating the db
@@ -806,9 +925,14 @@ class ClusterOps(BatchAPIClient):
         # Update the centroid collection
         self.model.vector_fields = vector_fields
 
-        print("Inserting centroid documents...")
         self._insert_centroid_documents()
 
+        print(
+            "Build your clustering app here: "
+            + f"https://cloud.relevance.ai/dataset/{self.dataset_id}/deploy/recent/cluster"
+        )
+
+    @track
     def fit_dataset(
         self,
         dataset,
@@ -849,19 +973,11 @@ class ClusterOps(BatchAPIClient):
             inplace=inplace,
         )
 
-    # def list_closest_to_center(self):
-    #     return self.datasets.cluster.centroids.list_closest_to_center(
-    #         dataset_id=self.dataset_id,
-    #         vector_fields=self.vector_fields
-    #         alias=self.alias
-    #     )
-
     def _concat_vectors_from_list(self, list_of_vectors: list):
         """Concatenate 2 vectors together in a pairwise fashion"""
         return [np.concatenate(x) for x in list_of_vectors]
 
     def _get_vectors_from_documents(self, vector_fields: list, documents: List[Dict]):
-
         if len(vector_fields) == 1:
             # filtering out entries not containing the specified vector
             documents = list(filter(DocUtils.list_doc_fields, documents))
@@ -892,6 +1008,7 @@ class ClusterOps(BatchAPIClient):
 
         return vectors
 
+    @track
     def partial_fit_documents(
         self,
         vector_fields: list,
@@ -923,20 +1040,16 @@ class ClusterOps(BatchAPIClient):
 
         .. code-block::
 
-            from relevanceai import ClusterBase, Client
-            import random
+            from relevanceai import Client
             client = Client()
+            df = client.Dataset("sample_dataset")
 
-            class CustomClusterModel(ClusterBase):
-                def fit_predict(self, X):
-                    cluster_labels = [random.randint(0, 100) for _ in range(len(X))]
-                    return cluster_labels
+            from sklearn.cluster import MiniBatchKMeans
+            model = MiniBatchKMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="batchkmeans_2", model=model)
 
-            model = CustomClusterModel()
-            clusterer = client.ClusterOps(model, alias="random")
-            df = client.Dataset("_github_repo_vectorai")
-
-            clusterer.parital_fit(df, vector_fields=["documentation_vector_"])
+            cluster_ops.parital_fit(df, vector_fields=["documentation_vector_"])
+            cluster_ops.predict_update(df, vector_fields=["sample_vector_"])
 
         """
         self.vector_fields = vector_fields
@@ -955,8 +1068,8 @@ class ClusterOps(BatchAPIClient):
         """Utility function for chunking a dataset"""
         cursor = None
 
-        docs = self.get_documents(
-            dataset.dataset_id,
+        docs = self._get_documents(
+            dataset_id=self.dataset_id,
             include_cursor=True,
             number_of_documents=chunksize,
             select_fields=select_fields,
@@ -965,8 +1078,8 @@ class ClusterOps(BatchAPIClient):
 
         while len(docs["documents"]) > 0:
             yield docs["documents"]
-            docs = self.get_documents(
-                self.dataset.dataset_id,
+            docs = self._get_documents(
+                dataset_id=self.dataset_id,
                 cursor=docs["cursor"],
                 include_cursor=True,
                 select_fields=select_fields,
@@ -974,6 +1087,7 @@ class ClusterOps(BatchAPIClient):
                 filters=filters,
             )
 
+    @track
     def partial_fit_dataset(
         self,
         dataset: Union[str, Dataset],
@@ -991,12 +1105,13 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample")
-            clusterer = client.ClusterOps(alias="minibatch_50", model=model)
-            clusterer.partial_fit_dataset(df, ["documentation_vector_"])
-            clusterer.predict_dataset(df, ['documentation_vector_'])
-            new_model.vector_fields = ['documentation_vector_']
-            clusterer.insert_centroid_documents(new_model.get_centroid_documents(), df)
+            df = client.Dataset("sample_dataset")
+
+            from sklearn.cluster import MiniBatchKMeans
+            model = MiniBatchKMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="minibatchkmeans_2", model=model)
+
+            cluster_ops.partial_fit_dataset(df, vector_fields=["documentation_vector_"])
 
         """
         self.vector_fields = vector_fields
@@ -1007,7 +1122,10 @@ class ClusterOps(BatchAPIClient):
 
         if isinstance(dataset, str):
             self.dataset = Dataset(
-                project=self.project, api_key=self.api_key, dataset_id=dataset
+                project=self.project,
+                api_key=self.api_key,
+                dataset_id=dataset,
+                firebase_uid=self.firebase_uid,
             )
         else:
             self.dataset = dataset
@@ -1028,6 +1146,7 @@ class ClusterOps(BatchAPIClient):
             vectors = self._get_vectors_from_documents(vector_fields, c)
             self.model.partial_fit(vectors)
 
+    @track
     def partial_fit_predict_update(
         self,
         dataset: Union[Dataset, str],
@@ -1059,8 +1178,12 @@ class ClusterOps(BatchAPIClient):
             from relevanceai import Client
             client = Client()
             df = client.Dataset("research2vec")
-            clusterer = client.ClusterOps(alias="minibatch_50", model=model)
-            clusterer.partial_fit_predict_update(
+
+            from sklearn.cluster import MiniBatchKMeans
+            model = MiniBatchKMeans(n_clusters=50)
+            cluster_ops = client.ClusterOps(alias="minibatchkmeans_50", model=model)
+
+            cluster_ops.partial_fit_predict_update(
                 df,
                 vector_fields=['title_trainedresearchqgen_vector_'],
                 chunksize=1000
@@ -1073,11 +1196,18 @@ class ClusterOps(BatchAPIClient):
         )
         print("Updating your dataset...")
         self.predict_update(dataset=dataset)
-        # if hasattr(self, "get_centers"):
-        #     print("Inserting your centroids...")
-        print("Inserting centroids...")
-        self.insert_centroid_documents(self.get_centroid_documents(), dataset=dataset)
+        if hasattr(self.model, "get_centers"):
+            print("Inserting your centroids...")
+            self.insert_centroid_documents(
+                self.get_centroid_documents(), dataset=dataset
+            )
 
+        print(
+            "Build your clustering app here: "
+            + f"https://cloud.relevance.ai/dataset/{self.dataset_id}/deploy/recent/cluster"
+        )
+
+    @track
     def predict_documents(
         self,
         vector_fields,
@@ -1100,6 +1230,7 @@ class ClusterOps(BatchAPIClient):
             return_only_clusters=return_only_clusters,
         )
 
+    @track
     def predict_update(
         self, dataset, vector_fields: Optional[List[str]] = None, chunksize: int = 20
     ):
@@ -1113,14 +1244,14 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample")
+            df = client.Dataset("sample_dataset")
 
-            from sklearn import MiniBatchKMeans
-            model = MiniBatchKMeans()
+            from sklearn.cluster import MiniBatchKMeans
+            model = MiniBatchKMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="minibatchkmeans_2", model=model)
 
-            clusterer = client.ClusterOps(model, alias="minibatch")
-            clusterer.partial_fit_dataset(df)
-            clusterer.predict_dataset(df)
+            cluster_ops.partial_fit_dataset(df)
+            cluster_ops.predict_dataset(df)
 
         """
         if not vector_fields:
@@ -1154,14 +1285,21 @@ class ClusterOps(BatchAPIClient):
                     all_responses["inserted"] += v
                 elif isinstance(all_responses[k], list):
                     all_responses[k] += v
+
+        print(
+            "Build your clustering app here: "
+            + f"https://cloud.relevance.ai/dataset/{self.dataset_id}/deploy/recent/cluster"
+        )
         return all_responses
 
+    @track
     def fit_predict_documents(
         self,
         vector_fields: list,
         documents: List[Dict],
         return_only_clusters: bool = True,
         inplace: bool = True,
+        include_grade: bool = False,
     ):
         """
         Train clustering algorithm on documents and then store the labels
@@ -1189,20 +1327,15 @@ class ClusterOps(BatchAPIClient):
 
         .. code-block::
 
-            from relevanceai import ClusterBase, Client
-            import random
+            from relevanceai import Client
             client = Client()
+            df = client.Dataset("sample_dataset")
 
-            class CustomClusterModel(ClusterBase):
-                def fit_predict(self, X):
-                    cluster_labels = [random.randint(0, 100) for _ in range(len(X))]
-                    return cluster_labels
+            from sklearn.cluster import MiniBatchKMeans
+            model = MiniBatchKMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="minibatchkmeans_2", model=model)
 
-            model = CustomClusterModel()
-            clusterer = client.ClusterOps(model, alias="random")
-            df = client.Dataset("_github_repo_vectorai")
-
-            clusterer.fit(df, vector_fields=["documentation_vector_"])
+            cluster_ops.fit_predict_documents(df, vector_fields=["documentation_vector_"])
 
         """
         self.vector_fields = vector_fields
@@ -1214,6 +1347,12 @@ class ClusterOps(BatchAPIClient):
         # Label the clusters
         cluster_labels = self._label_clusters(cluster_labels)
 
+        if include_grade:
+            try:
+                self._calculate_silhouette_grade(vectors, cluster_labels)
+            except Exception as e:
+                print(e)
+                pass
         return self.set_cluster_labels_across_documents(
             cluster_labels,
             documents,
@@ -1221,6 +1360,17 @@ class ClusterOps(BatchAPIClient):
             return_only_clusters=return_only_clusters,
         )
 
+    @staticmethod
+    def _calculate_silhouette_grade(vectors, cluster_labels):
+        from relevanceai.cluster_report.grading import get_silhouette_grade
+        from sklearn.metrics import silhouette_samples
+
+        score = silhouette_samples(vectors, cluster_labels, metric="euclidean").mean()
+        grade = get_silhouette_grade(score)
+        print(
+            f"You have received a grade of {grade} based on the mean silhouette score of {score}."
+        )
+
     def set_cluster_labels_across_documents(
         self,
         cluster_labels: list,
@@ -1252,7 +1402,7 @@ class ClusterOps(BatchAPIClient):
 
             labels = list(range(10))
             documents = [{"_id": str(x)} for x in range(10)]
-            clusterer.set_cluster_labels_across_documents(labels, documents)
+            cluster_ops.set_cluster_labels_across_documents(labels, documents)
 
         """
         if inplace:
@@ -1274,7 +1424,7 @@ class ClusterOps(BatchAPIClient):
             ]
         return new_documents
 
-    def _set_cluster_labels_across_documents(self, cluster_labels, documents):
+    def _get_cluster_field_name(self):
         if isinstance(self.vector_fields, list):
             set_cluster_field = (
                 f"{self.cluster_field}.{'.'.join(self.vector_fields)}.{self.alias}"
@@ -1283,6 +1433,10 @@ class ClusterOps(BatchAPIClient):
             set_cluster_field = (
                 f"{self.cluster_field}.{self.vector_fields}.{self.alias}"
             )
+        return set_cluster_field
+
+    def _set_cluster_labels_across_documents(self, cluster_labels, documents):
+        set_cluster_field = self._get_cluster_field_name()
         self.set_field_across_documents(set_cluster_field, cluster_labels, documents)
 
     def _label_cluster(self, label: Union[int, str]):
@@ -1342,13 +1496,14 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("_github_repo_vectorai")
-            from relevanceai.clusterer import KMeansModel
+            df = client.Dataset("sample_dataset")
 
-            model = KMeansModel()
-            kmeans = client.ClusterOps(model, alias="kmeans_sample")
-            kmeans.fit(df, vector_fields=["sample_1_vector_"])
-            kmeans.metadata
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
+
+            cluster_ops.fit(df, vector_fields=["sample_1_vector_"])
+            cluster_ops.metadata
             # {"k": 10}
 
         """
@@ -1359,6 +1514,7 @@ class ClusterOps(BatchAPIClient):
             metadata=metadata,
         )
 
+    @track
     def evaluate(
         self,
         ground_truth_column: Union[str, None] = None,
@@ -1391,15 +1547,16 @@ class ClusterOps(BatchAPIClient):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("_github_repo_vectorai")
-            from relevanceai.clusterer import KMeansModel
+            df = client.Dataset("sample_dataset")
 
-            model = KMeansModel()
-            kmeans = client.ClusterOps(model, alias="kmeans_sample")
-            kmeans.fit(df, vector_fields=["sample_1_vector_"])
+            from sklearn.cluster import KMeans
+            model = KMeans(n_clusters=2)
+            cluster_ops = client.ClusterOps(alias="kmeans_2", model=model)
 
-            kmeans.evaluate()
-            kmeans.evaluate("truth_column")
+            cluster_ops.fit(df, vector_fields=["sample_vector_"])
+
+            cluster_ops.evaluate()
+            cluster_ops.evaluate("truth_column")
         """
 
         from sklearn.metrics import (
@@ -1479,3 +1636,43 @@ class ClusterOps(BatchAPIClient):
                 }
 
         return stats
+
+    @track
+    def report(self):
+        """
+        Get a report on your clusters.
+
+        Example
+        ---------
+        .. code-block::
+
+            from relevanceai.datasets import mock_documents
+            docs = mock_documents(10)
+            df = client.Dataset('sample')
+            df.upsert_documents(docs)
+            cluster_ops = df.auto_cluster('kmeans-2', ['sample_1_vector_'])
+            cluster_ops.report()
+
+        """
+        if isinstance(self.vector_fields, list) and len(self.vector_fields) > 1:
+            raise ValueError(
+                "We currently do not support more than 1 vector field when reporting."
+            )
+        from relevanceai.cluster_report import ClusterReport
+
+        # X is all the vectors
+        cluster_field_name = self._get_cluster_field_name()
+        all_docs = self._get_all_documents(
+            self.dataset_id, select_fields=self.vector_fields + [cluster_field_name]
+        )
+        cluster_labels = self.get_field_across_documents(cluster_field_name, all_docs)
+        self.number_of_clusters = len(set(cluster_labels))
+        self._report = ClusterReport(
+            self.get_field_across_documents(self.vector_fields[0], all_docs),
+            cluster_labels=self.get_field_across_documents(
+                cluster_field_name, all_docs
+            ),
+            model=self.model,
+            num_clusters=self.number_of_clusters,
+        )
+        return self._report.internal_report
