@@ -3,23 +3,33 @@
 Pandas like dataset API
 """
 import warnings
-from typing import Dict, List, Optional
+import itertools
+from collections import Counter
+from typing import Dict, List, Optional, Callable
+from tqdm.auto import tqdm
+from relevanceai.analytics_funcs import track
 from relevanceai.dataset_api.dataset_write import Write
 from relevanceai.dataset_api.dataset_series import Series
+from relevanceai.data_tools.base_text_processing import MLStripper
 from relevanceai.vector_tools.nearest_neighbours import (
     NearestNeighbours,
     NEAREST_NEIGHBOURS,
 )
 
+from relevanceai.logger import FileLogger
+
 
 class Operations(Write):
+    @track
     def vectorize(self, field: str, model):
         """
         Vectorizes a Particular field (text) of the dataset
 
         .. warning::
             This function is currently in beta and is likely to change in the future.
-            We recommend not using this in any production systems.
+            We recommend not using this in any production systems. We recommend using
+            the bulk_apply function or the apply function to provide the intended output
+            for now.
 
         Parameters
         ------------
@@ -39,7 +49,7 @@ class Operations(Write):
 
             client = Client()
 
-            dataset_id = "sample_dataset"
+            dataset_id = "sample_dataset_id"
             df = client.Dataset(dataset_id)
 
             text_field = "text_field"
@@ -49,9 +59,11 @@ class Operations(Write):
             project=self.project,
             api_key=self.api_key,
             dataset_id=self.dataset_id,
+            firebase_uid=self.firebase_uid,
             field=field,
         ).vectorize(model)
 
+    @track
     def cluster(self, model, alias, vector_fields, **kwargs):
         """
         Performs KMeans Clustering on over a vector field within the dataset.
@@ -66,7 +78,6 @@ class Operations(Write):
         vector_fields : str
             The vector fields over which to cluster
 
-
         Example
         -------
         .. code-block::
@@ -77,7 +88,7 @@ class Operations(Write):
 
             client = Client()
 
-            dataset_id = "sample_dataset"
+            dataset_id = "sample_dataset_id"
             df = client.Dataset(dataset_id)
 
             vector_field = "vector_field_"
@@ -90,11 +101,16 @@ class Operations(Write):
         from relevanceai.clusterer import ClusterOps
 
         clusterer = ClusterOps(
-            model=model, alias=alias, api_key=self.api_key, project=self.project
+            model=model,
+            alias=alias,
+            api_key=self.api_key,
+            project=self.project,
+            firebase_uid=self.firebase_uid,
         )
         clusterer.fit_predict_update(dataset=self, vector_fields=vector_fields)
         return clusterer
 
+    @track
     def label_vector(
         self,
         vector,
@@ -151,7 +167,7 @@ class Operations(Write):
 
             client = Client()
 
-            dataset_id = "sample_dataset"
+            dataset_id = "sample_dataset_id"
             df = client.Dataset(dataset_id)
 
             result = df.label_vector(
@@ -184,8 +200,10 @@ class Operations(Write):
         # {"_label_": {"field": {"alias": [{"label": 3, "similarity_score": 0.4}]}
         return self.store_labels_in_document(labels, alias)
 
+    @track
     def store_labels_in_document(self, labels: list, alias: str):
-        # return {"_label_": {label_vector_field: {alias: labels}}}
+        if isinstance(labels, dict) and "label" in labels:
+            return {"_label_": {alias: labels["label"]}}
         return {"_label_": {alias: labels}}
 
     def _get_nearest_labels(
@@ -216,6 +234,7 @@ class Operations(Write):
             ]
         return new_labels
 
+    @track
     def label_document(
         self,
         document: dict,
@@ -270,7 +289,7 @@ class Operations(Write):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample_dataset")
+            df = client.Dataset("sample_dataset_id")
 
             results = df.label_document(
                 document={...},
@@ -304,7 +323,8 @@ class Operations(Write):
         document.update(self.store_labels_in_document(labels, alias))
         return document
 
-    def label(
+    @track
+    def label_from_dataset(
         self,
         vector_field: str,
         alias: str,
@@ -356,7 +376,7 @@ class Operations(Write):
 
             from relevanceai import Client
             client = Client()
-            df = client.Dataset("sample_dataset")
+            df = client.Dataset("sample_dataset_id")
 
             results = df.label(
                 vector_field="sample_1_vector_",
@@ -427,6 +447,506 @@ class Operations(Write):
             ],
         )
 
+    @track
+    def label_from_list(
+        self,
+        vector_field: str,
+        model: Callable,
+        label_list: list,
+        similarity_metric="cosine",
+        number_of_labels: int = 1,
+        score_field: str = "_search_score",
+        alias: Optional[str] = None,
+    ):
+        """Label from a given list.
+
+        Parameters
+        ------------
+
+        vector_field: str
+            The vector field to label in the original dataset
+        model: Callable
+            This will take a list of strings and then encode them
+        label_list: List
+            A list of labels to accept
+        similarity_metric: str
+            The similarity metric to accept
+        number_of_labels: int
+            The number of labels to accept
+        score_field: str
+            What to call the scoring of the labels
+        alias: str
+            The alias of the labels
+
+        Example
+        --------
+
+        .. code-block::
+
+            from relevanceai import Client
+            client = Client()
+            df = client.Dataset("sample")
+
+            # Get a model to help us encode
+            from vectorhub.encoders.text.tfhub import USE2Vec
+            enc = USE2Vec()
+
+            # Use that model to help with encoding
+            label_list = ["dog", "cat"]
+
+            df = client.Dataset("_github_repo_vectorai")
+
+            df.label_from_list("documentation_vector_", enc.bulk_encode, label_list, alias="pets")
+
+        """
+        if alias is None:
+            warnings.warn(
+                "No alias is detected for labelling. Default to 'default' as the alias."
+            )
+            alias = "default"
+        print("Encoding labels...")
+        label_vectors = []
+        for c in self.chunk(label_list, chunksize=20):
+            with FileLogger(verbose=True):
+                label_vectors.extend(model(c))
+
+        if len(label_vectors) == 0:
+            raise ValueError("Failed to encode.")
+
+        # we need this to mock label documents - these values are not important
+        # and can be changed :)
+        LABEL_VECTOR_FIELD = "label_vector_"
+        LABEL_FIELD = "label"
+
+        label_documents = [
+            {LABEL_VECTOR_FIELD: label_vectors[i], LABEL_FIELD: label}
+            for i, label in enumerate(label_list)
+        ]
+
+        return self._bulk_label_dataset(
+            label_documents=label_documents,
+            vector_field=vector_field,
+            label_vector_field=LABEL_VECTOR_FIELD,
+            similarity_metric=similarity_metric,
+            number_of_labels=number_of_labels,
+            score_field=score_field,
+            label_fields=[LABEL_FIELD],
+            alias=alias,
+        )
+
+    def _bulk_label_dataset(
+        self,
+        label_documents,
+        vector_field,
+        label_vector_field,
+        similarity_metric,
+        number_of_labels,
+        score_field,
+        label_fields,
+        alias,
+    ):
+        def label_and_store(d: dict):
+            labels = self._get_nearest_labels(
+                label_documents=label_documents,
+                vector=self.get_field(vector_field, d),
+                label_vector_field=label_vector_field,
+                similarity_metric=similarity_metric,
+                number_of_labels=number_of_labels,
+                score_field=score_field,
+                label_fields=label_fields,
+            )
+            d.update(self.store_labels_in_document(labels, alias))
+            return d
+
+        def bulk_label_documents(documents):
+            [label_and_store(d) for d in documents]
+            return documents
+
+        print("Labelling dataset...")
+        return self.bulk_apply(
+            bulk_label_documents,
+            filters=[
+                {
+                    "field": vector_field,
+                    "filter_type": "exists",
+                    "condition": ">=",
+                    "condition_value": " ",
+                },
+            ],
+            select_fields=[vector_field],
+        )
+
+    def _set_up_nltk(
+        self, stopwords_dict: str = "english", additional_stopwords: list = []
+    ):
+        """Additional stopwords to include"""
+        import nltk
+        from nltk.corpus import stopwords
+
+        self._is_set_up = True
+        nltk.download("stopwords")
+        nltk.download("punkt")
+        self.eng_stopwords = stopwords.words(stopwords_dict)
+        self.eng_stopwords.extend(additional_stopwords)
+        self.eng_stopwords = set(self.eng_stopwords)
+
+    def clean_html(self, html):
+        """Cleans HTML from text"""
+        s = MLStripper()
+        if html is None:
+            return ""
+        s.feed(html)
+        return s.get_data()
+
+    def get_word_count(self, text_fields: List[str]):
+        """
+        Create labels from a given text field.
+
+        Parameters
+        ------------
+
+        text_fields: list
+            List of text fields
+
+
+        Example
+        ------------
+
+        .. code-block::
+
+            from relevanceai import Client
+            client = Client()
+            df = client.Dataset("sample")
+            df.get_word_count()
+
+        """
+        import nltk
+        from nltk.corpus import stopwords
+
+        raise NotImplementedError
+
+    def generate_text_list_from_documents(
+        self, documents: list = [], text_fields: list = [], clean_html: bool = False
+    ):
+        """
+        Generate a list of text from documents to feed into the counter
+        model.
+        Parameters
+        -------------
+        documents: list
+            A list of documents
+        fields: list
+            A list of fields
+        clean_html: bool
+            If True, also cleans the text in a given text document to remove HTML. Will be slower
+            if processing on a large document
+        """
+        text = self.get_fields_across_documents(
+            text_fields, documents, missing_treatment="ignore"
+        )
+        return list(itertools.chain.from_iterable(text))
+
+    def generate_text_list(
+        self,
+        filters: list = [],
+        batch_size: int = 20,
+        text_fields: list = [],
+        cursor: str = None,
+    ):
+        filters += [
+            {
+                "field": tf,
+                "filter_type": "exists",
+                "condition": "==",
+                "condition_value": " ",
+            }
+            for tf in text_fields
+        ]
+        documents = self.get_documents(
+            batch_size=batch_size,
+            select_fields=text_fields,
+            filters=filters,
+            cursor=cursor,
+        )
+        return self.generate_text_list_from_documents(
+            documents, text_fields=text_fields
+        )
+
+    def get_ngrams(
+        self,
+        text,
+        n: int = 2,
+        stopwords_dict: str = "english",
+        additional_stopwords: list = [],
+        min_word_length: int = 2,
+        preprocess_hooks: list = [],
+    ):
+        try:
+            return self._get_ngrams(
+                text=text,
+                n=n,
+                additional_stopwords=additional_stopwords,
+                min_word_length=min_word_length,
+                preprocess_hooks=preprocess_hooks,
+            )
+        except:
+            # Specify that this shouldn't necessarily error out.
+            self.set_up(
+                stopwords_dict=stopwords_dict, additional_stopwords=additional_stopwords
+            )
+            return self._get_ngrams(
+                text=text,
+                n=n,
+                min_word_length=min_word_length,
+                preprocess_hooks=preprocess_hooks,
+            )
+
+    def _get_ngrams(
+        self,
+        text,
+        n: int = 2,
+        additional_stopwords=[],
+        min_word_length: int = 2,
+        preprocess_hooks: list = [],
+    ):
+        """Get the bigrams"""
+
+        from nltk import word_tokenize
+        from nltk.util import ngrams
+        from itertools import chain
+
+        if additional_stopwords:
+            [self.eng_stopwords.add(s) for s in additional_stopwords]
+
+        n_grams = []
+        for line in text:
+            for p_hook in preprocess_hooks:
+                line = p_hook(line)
+            token = word_tokenize(line)
+            n_grams.append(list(ngrams(token, n)))
+
+        def length_longer_than_min_word_length(x):
+            return len(x.strip()) >= min_word_length
+
+        def is_not_stopword(x):
+            return x.strip() not in self.eng_stopwords
+
+        def is_clean(text_list):
+            return all(
+                length_longer_than_min_word_length(x) and is_not_stopword(x)
+                for x in text_list
+            )
+
+        counter = Counter([" ".join(x) for x in chain(*n_grams) if is_clean(x)])
+        return counter
+        # return dict(counter.most_common(most_common))
+
+    @track
+    def get_wordcloud(
+        self,
+        text_fields: list,
+        n: int = 2,
+        most_common: int = 10,
+        filters: list = [],
+        additional_stopwords: list = [],
+        min_word_length: int = 2,
+        batch_size: int = 1000,
+        document_limit: int = None,
+        preprocess_hooks: List[callable] = [],
+    ) -> list:
+        """
+        wordcloud return object looks like:
+
+        .. code-block::
+
+            {'python': 232}
+
+        Parameters
+        ------------
+
+        text_fields: list
+            A list of text fields
+
+        Example
+        ----------
+
+        .. code-block::
+
+            from relevanceai import Client
+            client = Client()
+
+        """
+        counter: Counter = Counter()
+        if not hasattr(self, "_is_set_up"):
+            print("setting up NLTK...")
+            self._set_up_nltk()
+
+        # Mock a dummy documents so I can loop immediately without weird logic
+        documents: dict = {"documents": [[]], "cursor": None}
+        print("Updating word count...")
+        while len(documents["documents"]) > 0 and (
+            document_limit is None or sum(counter.values()) < document_limit
+        ):
+            documents = self.get_documents(
+                filters=filters,
+                cursor=documents["cursor"],
+                batch_size=batch_size,
+                select_fields=text_fields,
+                include_cursor=True,
+            )
+            string = self.generate_text_list_from_documents(
+                documents=documents["documents"],
+                text_fields=text_fields,
+            )
+
+            ngram_counter = self._get_ngrams(
+                string,
+                n=n,
+                additional_stopwords=additional_stopwords,
+                min_word_length=min_word_length,
+                preprocess_hooks=preprocess_hooks,
+            )
+            counter.update(ngram_counter)
+        return counter.most_common(most_common)
+
+    def cluster_word_cloud(
+        self,
+        vector_fields: List[str],
+        text_fields: List[str],
+        cluster_alias: str,
+        n: int = 2,
+        cluster_field: str = "_cluster_",
+        num_clusters: int = 100,
+        preprocess_hooks: List[callable] = [],
+    ):
+        """
+        Simple implementation of the cluster word cloud
+        """
+        vector_fields_str = ".".join(sorted(vector_fields))
+        field = f"{cluster_field}.{vector_fields_str}.{cluster_alias}"
+        all_clusters = self.facets([field], page_size=num_clusters)
+        most_common = 10
+        cluster_counters = {}
+        for c in tqdm(all_clusters[field]):
+            cluster_value = c[field]
+            top_words = self.get_wordcloud(
+                text_fields=text_fields,
+                n=n,
+                filters=[
+                    {
+                        "field": field,
+                        "filter_type": "contains",
+                        "condition": "==",
+                        "condition_value": cluster_value,
+                    }
+                ],
+                most_common=most_common,
+                preprocess_hooks=preprocess_hooks,
+            )
+            cluster_counters[c] = top_words
+        return cluster_counters
+
+    def _add_cluster_word_cloud_to_config(self, data, cluster_value, top_words):
+        # hacky way I implemented to add top words to config
+        data["configuration"]["cluster-labels"][cluster_value] = ", ".join(
+            [k for k in top_words if k != "machine learning"]
+        )
+        data["configuration"]["cluster-descriptions"][cluster_value] = str(top_words)
+
+    @track
+    def label_from_common_words(
+        self,
+        text_field: str,
+        model: Callable = None,
+        most_common: int = 1000,
+        n_gram: int = 1,
+        temp_vector_field: str = "_label_vector_",
+        labels_fn="labels.txt",
+        stopwords: list = [],
+    ):
+        """
+        Label by the most popular keywords.
+
+        Algorithm:
+
+        - Get top X keywords or bigram for a text field
+        - Default X to 1000 or something scaled towards number of documents
+        - Vectorize those into keywords
+        - Label every document with those top keywords
+
+        .. note::
+            **New in v1.1.0**
+
+        Parameters
+        ------------
+
+        text_fields: str
+            The field to label
+        model: Callable
+            The function or callable to turn text into a vector.
+        most_common: int
+            How many of the most common worsd do you want to use as labels
+        n_gram: int
+            How many word co-occurrences do you want to consider
+        temp_vector_field: str
+            The temporary vector field name
+        labels_fn: str
+            The filename for labels to be saved in.
+        stopwords: list
+            A list of stopwords
+
+        Example
+        --------
+
+        .. code-block::
+
+            import random
+            from relevanceai import Client
+            from relevanceai.datasets import mock_documents
+            from relevanceai.logger import FileLogger
+
+            client = Client()
+            ds = client.Dataset("sample")
+            documents = mock_documents()
+            ds.insert_documents(documents)
+
+            def encode():
+                return [random.randint(0, 100) for _ in range(5)]
+
+            ds.label_from_common_words(
+                text_field="sample_1_label",
+                model=encode,
+                most_common=10,
+                n_gram=1
+            )
+
+        """
+        labels = self.get_wordcloud(
+            text_fields=[text_field],
+            n=n_gram,
+            most_common=most_common,
+            additional_stopwords=stopwords,
+        )
+
+        with open(labels_fn, "w") as f:
+            f.write(str(labels))
+
+        print(f"Saved labels to {labels_fn}")
+        # Add support if already encoded
+        def encode(doc):
+            with FileLogger():
+                doc[temp_vector_field] = model(self.get_field(text_field, doc))
+            return doc
+
+        self.apply(encode, select_fields=[text_field])
+        # create_labels_from_text_field
+        return self.label_from_list(
+            vector_field=temp_vector_field,
+            model=model,
+            label_list=[x[0] for x in labels],
+        )
+
+    @track
     def vector_search(
         self,
         multivector_query: List,
@@ -578,6 +1098,7 @@ class Operations(Write):
             query=query,
         )
 
+    @track
     def hybrid_search(
         self,
         multivector_query: List,
@@ -694,6 +1215,7 @@ class Operations(Write):
             search_history_id=search_history_id,
         )
 
+    @track
     def chunk_search(
         self,
         multivector_query,
@@ -812,6 +1334,7 @@ class Operations(Write):
             query=query,
         )
 
+    @track
     def multistep_chunk_search(
         self,
         multivector_query,
@@ -933,6 +1456,51 @@ class Operations(Write):
             query=query,
         )
 
+    def _run_dr_algorithm(
+        self,
+        algorithm: str,
+        vector_fields: list,
+        documents: list,
+        alias: str,
+        n_components: int = 3,
+    ):
+        original_name = algorithm
+        # Make sure that the letter case does not matter
+        algorithm = algorithm.upper()
+        if algorithm == "PCA":
+            from relevanceai.vector_tools.dim_reduction import PCA
+
+            model = PCA()
+        elif algorithm == "TSNE":
+            from relevanceai.vector_tools.dim_reduction import TSNE
+
+            model = TSNE()
+        elif algorithm == "UMAP":
+            from relevanceai.vector_tools.dim_reduction import UMAP
+
+            model = UMAP()
+        elif algorithm == "IVIS":
+            from relevanceai.vector_tools.dim_reduction import Ivis
+
+            model = Ivis()
+        else:
+            raise ValueError(
+                f'"{original_name}" is not a supported '
+                "dimensionality reduction algorithm. "
+                "Currently, the supported algorithms are: "
+                "PCA, TSNE, UMAP, and IVIS"
+            )
+
+        print(f"Run {algorithm}...")
+        # Returns a list of documents with dr vector
+        return model.fit_transform_documents(
+            vector_field=vector_fields[0],
+            documents=documents,
+            alias=alias,
+            dims=n_components,
+        )
+
+    @track
     def auto_reduce_dimensions(
         self,
         alias: str,
@@ -943,16 +1511,11 @@ class Operations(Write):
         """
         Run dimensionality reduction quickly on a dataset on a small number of documents.
         This is useful if you want to quickly see a projection of your dataset.
-        Currently, the only supported algorithm is `PCA`.
 
         .. warning::
             This function is currently in beta and is likely to change in the future.
             We recommend not using this in any production systems.
 
-
-        .. note::
-            **New in v0.32.0**
-
         Parameters
         ----------
         vector_fields: list
@@ -1007,31 +1570,28 @@ class Operations(Write):
             number_of_documents = self.get_number_of_documents(self.dataset_id, filters)
 
         documents = self.get_documents(
-            dataset_id=self.dataset_id,
             select_fields=vector_fields,
             filters=filters,
             number_of_documents=number_of_documents,
         )
 
-        print("Run PCA...")
-        if algorithm == "pca":
-            dr_docs = self._run_pca(
-                vector_fields=vector_fields,
-                documents=documents,
-                alias=alias,
-                n_components=n_components,
-            )
-        else:
-            raise ValueError("DR algorithm not supported.")
+        dr_documents = self._run_dr_algorithm(
+            algorithm=algorithm,
+            vector_fields=vector_fields,
+            documents=documents,
+            alias=alias,
+            n_components=n_components,
+        )
 
-        results = self.update_documents(self.dataset_id, dr_docs)
+        results = self.update_documents(self.dataset_id, dr_documents)
 
         if n_components == 3:
             projector_url = f"https://cloud.relevance.ai/dataset/{self.dataset_id}/deploy/recent/projector"
-            print(f"You can now view your {projector_url}")
+            print(f"You can now view your projector at {projector_url}")
 
         return results
 
+    @track
     def reduce_dimensions(
         self,
         vector_fields: list,
@@ -1044,16 +1604,11 @@ class Operations(Write):
         """
         Run dimensionality reduction quickly on a dataset on a small number of documents.
         This is useful if you want to quickly see a projection of your dataset.
-        Currently, the only supported algorithm is `PCA`.
 
         .. warning::
             This function is currently in beta and is likely to change in the future.
             We recommend not using this in any production systems.
 
-
-        .. note::
-            **New in v0.32.0**
-
         Parameters
         ----------
         vector_fields: list
@@ -1097,44 +1652,24 @@ class Operations(Write):
             for vf in vector_fields
         ]
         documents = self.get_documents(
-            dataset_id=self.dataset_id,
             select_fields=vector_fields,
             filters=filters,
             number_of_documents=number_of_documents,
         )
 
-        print("Run PCA...")
-        if algorithm == "pca":
-            dr_docs = self._run_pca(
-                vector_fields=vector_fields,
-                documents=documents,
-                alias=alias,
-                n_components=n_components,
-            )
+        dr_documents = self._run_dr_algorithm(
+            algorithm=algorithm,
+            vector_fields=vector_fields,
+            documents=documents,
+            alias=alias,
+            n_components=n_components,
+        )
 
-        else:
-            raise ValueError(
-                "DR algorithm not supported. Only supported algorithms are `pca`."
-            )
-
-        results = self.update_documents(self.dataset_id, dr_docs)
+        results = self.update_documents(self.dataset_id, dr_documents)
 
         return results
 
-    def _run_pca(
-        self, vector_fields: list, documents: list, alias: str, n_components: int = 3
-    ):
-        from relevanceai.vector_tools.dim_reduction import PCA
-
-        model = PCA()
-        # Returns a list of documents with the dr vector
-        return model.fit_transform_documents(
-            vector_field=vector_fields[0],
-            documents=documents,
-            alias=alias,
-            dims=n_components,
-        )
-
+    @track
     def auto_cluster(self, alias: str, vector_fields: List[str], chunksize: int = 1024):
         """
         Automatically cluster in 1 line of code.
@@ -1167,7 +1702,7 @@ class Operations(Write):
 
             client = Client()
 
-            dataset_id = "sample_dataset"
+            dataset_id = "sample_dataset_id"
             df = client.Dataset(dataset_id)
 
             # run kmeans with default 10 clusters
@@ -1194,9 +1729,9 @@ class Operations(Write):
         if n_clusters >= chunksize:
             raise ValueError("Number of clustesr exceed chunksize.")
 
-        num_docs = self.get_number_of_documents(self.dataset_id)
+        num_documents = self.get_number_of_documents(self.dataset_id)
 
-        if num_docs <= n_clusters:
+        if num_documents <= n_clusters:
             warnings.warn(
                 "You seem to have more clusters than documents. We recommend reducing the number of clusters."
             )
@@ -1212,10 +1747,13 @@ class Operations(Write):
                 alias=alias,
                 api_key=self.api_key,
                 project=self.project,
+                firebase_uid=self.firebase_uid,
                 dataset_id=self.dataset_id,
                 vector_fields=vector_fields,
             )
-            clusterer.fit_predict_update(dataset=self, vector_fields=vector_fields)
+            clusterer.fit_predict_update(
+                dataset=self, vector_fields=vector_fields, include_grade=True
+            )
 
         elif algorithm.lower() == "hdbscan":
             raise ValueError(
@@ -1231,6 +1769,7 @@ class Operations(Write):
                 alias=alias,
                 api_key=self.api_key,
                 project=self.project,
+                firebase_uid=self.firebase_uid,
                 dataset_id=self.dataset_id,
                 vector_fields=vector_fields,
             )
@@ -1241,9 +1780,47 @@ class Operations(Write):
         else:
             raise ValueError("Only KMeans clustering is supported at the moment.")
 
-        # Get users excited about being able to build a dashboard!
-        print(
-            "Build your clustering app here: "
-            + f"https://cloud.relevance.ai/dataset/{self.dataset_id}/deploy/recent/cluster"
-        )
         return clusterer
+
+    @track
+    def aggregate(
+        self,
+        groupby: list = [],
+        metrics: list = [],
+        filters: list = [],
+        sort: list = [],
+        page_size: int = 20,
+        page: int = 1,
+        asc: bool = False,
+        flatten: bool = True,
+        alias: str = "default",
+    ):
+        return self.services.aggregate.aggregate(
+            dataset_id=self.dataset_id,
+            groupby=groupby,
+            metrics=metrics,
+            filters=filters,
+            page_size=page_size,
+            page=page,
+            asc=asc,
+            flatten=flatten,
+            alias=alias,
+            # sort=sort
+        )
+
+    def facets(
+        self,
+        fields: list = [],
+        date_interval: str = "monthly",
+        page_size: int = 5,
+        page: int = 1,
+        asc: bool = False,
+    ):
+        return self.datasets.facets(
+            dataset_id=self.dataset_id,
+            fields=fields,
+            date_interval=date_interval,
+            page_size=page_size,
+            page=page,
+            asc=asc,
+        )
