@@ -5,6 +5,9 @@ These datasets have been licensed under Apache 2.0.
 """
 
 from typing import List, Union, Dict, Any
+import random
+import uuid
+import string
 import sys
 import pandas as pd
 import requests
@@ -61,15 +64,13 @@ class ExampleDatasets:
         api_key = (
             "UzdYRktIY0JxNmlvb1NpOFNsenU6VGdTU0s4UjhUR0NsaDdnQTVwUkpKZw"  # read access
         )
-        client = Client(
-            project,
-            api_key,
-        )
-        documents = client.get_documents(
+        client = Client(project, api_key, region="old-australia-east")
+        documents = client._get_documents(
             db_name,
             number_of_documents=number_of_documents,
             select_fields=select_fields,
         )
+        client.config.reset()
         return documents
 
     @staticmethod
@@ -503,6 +504,89 @@ def get_machine_learning_research_dataset():
     raise NotImplementedError
 
 
+def mock_documents(number_of_documents: int = 100, vector_length=5):
+    """
+    Utility function to mock documents. Aimed at helping users reproduce errors
+    if required.
+    The schema for the documents is as follows:
+
+    .. code-block::
+
+        {'_chunk_': 'chunks',
+        '_chunk_.label': 'text',
+        '_chunk_.label_chunkvector_': {'chunkvector': 5},
+        'insert_date_': 'date',
+        'sample_1_description': 'text',
+        'sample_1_label': 'text',
+        'sample_1_value': 'numeric',
+        'sample_1_vector_': {'vector': 5},
+        'sample_2_description': 'text',
+        'sample_2_label': 'text',
+        'sample_2_value': 'numeric',
+        'sample_2_vector_': {'vector': 5},
+        'sample_3_description': 'text',
+        'sample_3_label': 'text',
+        'sample_3_value': 'numeric',
+        'sample_3_vector_': {'vector': 5}}
+
+    Parameters
+    ------------
+
+    number_of_documents: int
+        The number of documents to mock
+    vector_length: int
+        The length of vectors
+
+    .. code-block::
+
+        from relevanceai.datasets import mock_documents
+        documents = mock_documents(10)
+
+    """
+
+    def generate_random_string(string_length: int = 5) -> str:
+        """Generate a random string of letters and numbers"""
+        return "".join(
+            random.choice(string.ascii_uppercase + string.digits)
+            for _ in range(string_length)
+        )
+
+    def generate_random_vector(vector_length: int = vector_length) -> List[float]:
+        """Generate a random list of floats"""
+        return [random.random() for _ in range(vector_length)]
+
+    def generate_random_label(label_value: int = 5) -> str:
+        return f"label_{random.randint(0, label_value)}"
+
+    def generate_random_integer(min: int = 0, max: int = 100) -> int:
+        return random.randint(min, max)
+
+    def vector_document(_id: str) -> Dict:
+        return {
+            "_id": _id,
+            "sample_1_label": generate_random_label(),
+            "sample_2_label": generate_random_label(),
+            "sample_3_label": generate_random_label(),
+            "sample_1_description": generate_random_string(),
+            "sample_2_description": generate_random_string(),
+            "sample_3_description": generate_random_string(),
+            "sample_1_vector_": generate_random_vector(),
+            "sample_2_vector_": generate_random_vector(),
+            "sample_3_vector_": generate_random_vector(),
+            "sample_1_value": generate_random_integer(),
+            "sample_2_value": generate_random_integer(),
+            "sample_3_value": generate_random_integer(),
+            "_chunk_": [
+                {
+                    "label": generate_random_label(),
+                    "label_chunkvector_": generate_random_vector(),
+                }
+            ],
+        }
+
+    return [vector_document(uuid.uuid4().__str__()) for _ in range(number_of_documents)]
+
+
 ### For backwards compatability
 
 get_ecommerce_1_dataset = get_dummy_ecommerce_dataset = get_ecommerce_dataset_encoded
