@@ -7,10 +7,12 @@ doc_utils = DocUtils()
 
 
 class NearestNeighbours(_Base, DocUtils):
-    def __init__(self, project: str, api_key: str):
+    def __init__(self, project: str, api_key: str, firebase_uid: str):
         self.project = project
         self.api_key = api_key
-        super().__init__(project, api_key)
+        self.firebase_uid = firebase_uid
+
+        super().__init__(project=project, api_key=api_key, firebase_uid=firebase_uid)
 
     @staticmethod
     def get_nearest_neighbours(
